var EXPORTED_SYMBOLS = [ "fburl" ];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://calendar/modules/calUtils.jsm");
Cu.import("resource://calendar/modules/calIteratorUtils.jsm");
Cu.import("resource:///modules/mailServices.js");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

if ("undefined" == typeof(fburl)) {
    var fburl = {};
};

const calIFreeBusyInterval = Ci.calIFreeBusyInterval;
const calIDateTime = Ci.calIDateTime;

fburl.fbUrlProvider = function(cal) {
    this.cal = cal;
    this._query = null;
    this._context = null;
    this._result = null;
    this._searched = false;
    this._error = null;
    this._listener = null;
    this._url = null;
    this._calId = null;
    this._periods = null;
};

fburl.fbUrlProvider.prototype = {
    classID: Components.ID("{11291d94-b457-4322-bfba-ae9df4b6a3c1}"),

    getFreeBusyIntervals: function (aCalId, aRangeStart, aRangeEnd, aBusyTypes, aListener) {

        this._listener = aListener;
        this._calId = aCalId;
        this._periods = null;
        this.startSearch(aCalId);
    },

    getLDAPAddressBooks: function () {

        var abManager = Cc["@mozilla.org/abmanager;1"].getService(Ci.nsIAbManager);
        var allAddressBooks = abManager.directories;
        var books = [];

        while (allAddressBooks.hasMoreElements()) {
            var addressBook = allAddressBooks.getNext().QueryInterface(Ci.nsIAbDirectory);
            if (addressBook instanceof Ci.nsIAbLDAPDirectory) {
                books.push(addressBook);
            }
        }

        return books;
    },

    startSearch: function (aCalId) {

        var service = Cc["@mozilla.org/network/ldap-service;1"].getService(Ci.nsILDAPService);
        var books = this.getLDAPAddressBooks();

        this._result = null;
        aCalId = aCalId.replace(/^mailto:/i, "");

        for each (var book in books) {
            var ldap = book.QueryInterface(Ci.nsIAbLDAPDirectory);
            var attrmap = Cc["@mozilla.org/addressbook/ldap-attribute-map;1"].createInstance(Ci.nsIAbLDAPAttributeMap);
            var args = Cc["@mozilla.org/addressbook/directory/query-arguments;1"].createInstance(Ci.nsIAbDirectoryQueryArguments);

            attrmap.setAttributeList("PrimaryEmail", "mail", true);
            attrmap.setAttributeList("FBURL", "calFbUrl", true);
            args.filter = service.createFilter(aCalId.length * 2, "(%a=%v)", "", "", "mail", aCalId);
            args.typeSpecificArg = attrmap;
            args.querySubDirectories = true;

            this._query = Cc["@mozilla.org/addressbook/ldap-directory-query;1"].createInstance(Ci.nsIAbDirectoryQuery);
            this._context = this._query.doQuery(ldap, args, this, ldap.maxHits, 0);
            this._searched = true;

            // NOTE: Do only one query for now.

            break;
        }
    },

    stopSearch: function stopSearch() {
        if (this._context) {
            this._query.stopQuery(this._context);
            this._context = null;
            this._listener = null;
        }
    },

    onSearchFinished: function (aResult, aErrorMsg) {
        this._context = null;
        this._error = aErrorMsg;
    },

    onSearchFoundCard: function (aCard) {

        this._result = aCard;

        // Obtain the URL from the card.

        this._url = aCard.getProperty("FBURL", "");

        // Retrieve the resource and parse it.

        var parser = Cc["@mozilla.org/calendar/ics-service;1"].getService(Ci.calIICSService);
        //var parser = Cc["@mozilla.org/calendar/ics-parser;1"].createInstance(Ci.calIIcsParser);
        var service = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
        var stream = Cc["@mozilla.org/scriptableinputstream;1"].createInstance(Ci.nsIScriptableInputStream);
        var channel = service.newChannelFromURI(service.newURI(this._url, null, null));

        stream.init(channel.open());
        var resource = parser.parseICS(stream.read(-1), null);

        // Obtain the free/busy periods.

        this._periods = [];
        var fbTypeMap = {};

        fbTypeMap["FREE"] = calIFreeBusyInterval.FREE;
        fbTypeMap["BUSY"] = calIFreeBusyInterval.BUSY;
        fbTypeMap["BUSY-UNAVAILABLE"] = calIFreeBusyInterval.BUSY_UNAVAILABLE;
        fbTypeMap["BUSY-TENTATIVE"] = calIFreeBusyInterval.BUSY_TENTATIVE;

        // Iterate over components in the response.

        for (var comp in cal.ical.calendarComponentIterator(resource)) {

            // Iterate over free/busy properties.

            for (var fbProp in cal.ical.propertyIterator(comp, "FREEBUSY")) {

                // Assign the stated type or busy otherwise.

                var fbType = fbProp.getParameter("FBTYPE");

                if (fbType) {
                    fbType = fbTypeMap[fbType];
                } else {
                    fbType = calIFreeBusyInterval.BUSY;
                }

                var parts = fbProp.value.split("/");
                var begin = cal.createDateTime(parts[0]);
                var end;

                // Support durations.

                if (parts[1].charAt(0) == "P") {
                    end = begin.clone();
                    end.addDuration(cal.createDuration(parts[1]));

                // Support plain datetimes.

                } else {
                    end = cal.createDateTime(parts[1]);
                }

                interval = new cal.FreeBusyInterval(this._calId, fbType, begin, end);
                this._periods.push(interval);
            }
        }

        this._listener.onResult(null, this._periods);
    },

    QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver, Ci.nsIAbDirSearchListener])
};

fburl.initProvider = function() {
    cal.getFreeBusyService().addProvider(new fburl.fbUrlProvider(cal));
}

fburl.test = function(aCalId) {
    var fbp = new fburl.fbUrlProvider(cal);
    fbp.startSearch(aCalId);
    return fbp;
}

const NSGetFactory = XPCOMUtils.generateNSGetFactory([fburl.fbUrlProvider]);
